@@ -14,7 +14,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: md-mx-ctrl.c 11287 2006-04-27 15:17:02Z dfs $";
+"$Id: md-mx-ctrl.c 13975 2007-02-09 18:04:24Z dfs $";
 
 #include "config.h"
 #define SMALLBUF 16384
@@ -68,7 +68,7 @@ percent_decode(unsigned char *buf)
 
     while(*in) {
 	if (*in == '%' && isxdigit(*(in+1)) && isxdigit(*(in+2))) {
-	    sscanf(in+1, "%2x", &val);
+	    sscanf((char *) in+1, "%2x", &val);
 	    *out++ = (unsigned char) val;
 	    in += 3;
 	    continue;
@@ -173,7 +173,7 @@ buildCmd(int argc, char *argv[], char *outbuf, int buflen)
     int k;
     int len = 0;
     for (i=0; i<argc; i++) {
-	k = percent_encode(argv[i], outbuf+len, buflen-len);
+	k = percent_encode((unsigned char *) argv[i], (unsigned char *) outbuf+len, buflen-len);
 	len += k;
 	if (len >= buflen-1) {
 	    outbuf[buflen-1] = 0;
@@ -236,7 +236,7 @@ doCmd(char const *sock, char const *cmd, int decode)
 	return EXIT_FAILURE;
     }
     if (decode) {
-	percent_decode(ans);
+	percent_decode((unsigned char *)ans);
     }
     printf("%s", ans);
     return EXIT_SUCCESS;
