@@ -8,7 +8,7 @@
 *
 ***********************************************************************/
 
-/* $Id: mimedefang.h 10190 2006-02-01 17:30:53Z dfs $ */
+/* $Id: mimedefang.h 13923 2007-02-02 20:37:05Z dfs $ */
 
 #ifndef MIMEDEFANG_H
 #define MIMEDEFANG_H 1
@@ -18,10 +18,10 @@
 #include <stddef.h>
 #include <sys/types.h>
 #include <sys/socket.h>   /* For struct sockaddr */
+
 extern void chomp(char *str);
-extern void write_percent_encoded(unsigned char const *str, int fd);
-extern int percent_encode(unsigned char *in, unsigned char *out, int outlen);
-extern void percent_decode(unsigned char *buf);
+extern int percent_encode(char const *in, char *out, int outlen);
+extern void percent_decode(char *buf);
 
 extern int MXCheckFreeSlaves(char const *sockname);
 extern int MXScanDir(char const *sockname, char const *dir);
@@ -41,7 +41,7 @@ extern int MXRecipientOK(char const *sockname, char *msg,
 			 char const *rcpt_mailer, char const *rcpt_host,
 			 char const *rcpt_addr);
 
-extern int safeWriteHeader(int fd, unsigned char *str);
+extern int safeWriteHeader(int fd, char *str);
 extern void split_on_space(char *buf, char **first, char **rest);
 extern void split_on_space3(char *buf,
 			    char **first, char **second, char **rest);
@@ -50,11 +50,10 @@ extern void split_on_space4(char *buf,
 			    char **rest);
 extern void *malloc_with_log(size_t s);
 extern char *strdup_with_log(char const *s);
-extern void write_mx_command(int fd, char cmd, unsigned char const *buf);
-extern int rm_r(char const *dir);
-extern int writen(int fd, unsigned char const *buf, size_t len);
+extern int rm_r(void *ctx, char const *dir);
+extern int writen(int fd, char const *buf, size_t len);
 extern int readn(int fd, void *buf, size_t count);
-extern int writestr(int fd, unsigned char const *buf);
+extern int writestr(int fd, char const *buf);
 extern int closefd(int fd);
 extern int validate_smtp_code(char const *code, char const *first);
 extern int validate_smtp_dsn(char const *dsn, char const *first);
@@ -62,6 +61,7 @@ extern int make_listening_socket(char const *str, int backlog, int must_be_unix)
 extern int connect_to_socket(char const *str);
 extern void do_delay(char const *sleepstr);
 extern int is_localhost(struct sockaddr *);
+extern int remove_local_socket(char const *str);
 #ifdef EMBED_PERL
 extern int make_embedded_interpreter(char const *progPath,
 				     char const *subFilter,
