@@ -13,13 +13,14 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: notifier.c 10090 2005-02-08 16:04:40Z dfs $";
+"$Id: notifier.c 13476 2006-12-15 17:08:10Z dfs $";
 
 #include "config.h"
 #include "event_tcp.h"
 #include "mimedefang.h"
 #include <syslog.h>
 #include <string.h>
+#include <stdio.h>
 
 #define MAX_LISTENERS 5
 #define MAX_MSGLEN 256
@@ -337,3 +338,37 @@ notify_listeners(EventSelector *es, char const *msg)
     }
 }
 
+/**********************************************************************
+* %FUNCTION: notify_slave_status
+* %ARGUMENTS:
+*  es -- event selector
+*  slaveno -- slave number
+*  msg -- slave's new status
+* %DESCRIPTION:
+*  Notifies all listeners of a message
+* %RETURNS:
+*  Nothing
+***********************************************************************/
+void
+notify_slave_status(EventSelector *es, int slaveno, char const *msg)
+{
+    char buf[256];
+
+    if (!ListenersInitialized) return;
+    if (!msg || !*msg) {
+	return;
+    }
+    snprintf(buf, sizeof(buf), "S %d %.200s\n", slaveno, msg);
+    notify_listeners(es, buf);
+}
+
+void
+notify_slave_state_change(EventSelector *es, int slaveno,
+			  char const *old_state, char const *new_state)
+{
+    char buf[256];
+    if (!ListenersInitialized) return;
+    snprintf(buf, sizeof(buf), "S %d StateChange %s -> %s\n",
+	     slaveno, old_state, new_state);
+    notify_listeners(es, buf);
+}
