@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: rm_r.c 10090 2005-02-08 16:04:40Z dfs $";
+"$Id: rm_r.c 12891 2006-11-02 14:35:49Z dfs $";
 
 #include "config.h"
 #include "mimedefang.h"
@@ -31,6 +31,21 @@ static char const RCSID[] =
 #include <sys/wait.h>
 #endif
 
+
+#ifdef ENABLE_DEBUGGING
+extern void *malloc_debug(void *, size_t, char const *fname, int);
+extern char *strdup_debug(void *, char const *, char const *, int);
+extern void free_debug(void *, void *, char const *, int);
+#undef malloc
+#undef strdup
+#undef free
+#define malloc(x) malloc_debug(ctx, x, __FILE__, __LINE__)
+#define strdup(x) strdup_debug(ctx, x, __FILE__, __LINE__)
+#define free(x) free_debug(ctx, x, __FILE__, __LINE__)
+#define malloc_with_log(x) malloc_debug(ctx, x, __FILE__, __LINE__)
+#define strdup_with_log(x) strdup_debug(ctx, x, __FILE__, __LINE__)
+#endif
+
 #ifndef EXEC_RM_FOR_CLEANUP
 /**********************************************************************
 * %FUNCTION: rm_r
@@ -42,7 +57,7 @@ static char const RCSID[] =
 *  Deletes dir and recursively deletes contents
 ***********************************************************************/
 int
-rm_r(char const *dir)
+rm_r(void *ctx, char const *dir)
 {
     char buf[SMALLBUF];
     struct stat sbuf;
@@ -101,7 +116,7 @@ rm_r(char const *dir)
 	    continue;
 	}
 	snprintf(buf, sizeof(buf), "%s/%s", dir, entry->d_name);
-	if (rm_r(buf) < 0) {
+	if (rm_r(ctx, buf) < 0) {
 	    int errno_save = errno;
 	    closedir(d);
 	    free(entry);
