@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 static char const RCSID[] =
-"$Id: utils.c 10178 2006-01-20 02:25:51Z dfs $";
+"$Id: utils.c 13974 2007-02-09 16:29:23Z dfs $";
 
 #define _BSD_SOURCE 1
 
@@ -43,6 +43,20 @@ static char const RCSID[] =
 #include <stdint.h>
 #endif
 
+#ifdef ENABLE_DEBUGGING
+extern void *malloc_debug(void *, size_t, char const *fname, int);
+extern char *strdup_debug(void *, char const *, char const *, int);
+extern void free_debug(void *, void *, char const *, int);
+#undef malloc
+#undef strdup
+#undef free
+#define malloc(x) malloc_debug(ctx, x, __FILE__, __LINE__)
+#define strdup(x) strdup_debug(ctx, x, __FILE__, __LINE__)
+#define free(x) free_debug(ctx, x, __FILE__, __LINE__)
+#define malloc_with_log(x) malloc_debug(ctx, x, __FILE__, __LINE__)
+#define strdup_with_log(x) strdup_debug(x, __FILE__, __LINE__)
+#endif
+
 #ifndef HAVE_UINT32_T
 /* On these machines, punt to unsigned int */
 typedef unsigned int uint32_t;
@@ -166,6 +180,7 @@ split_on_space4(char *buf,
     }
 }
 
+#ifndef ENABLE_DEBUGGING
 /**********************************************************************
 * %FUNCTION: malloc_with_log
 * %ARGUMENTS:
@@ -205,6 +220,7 @@ strdup_with_log(char const *s)
     }
     return p;
 }
+#endif
 
 /**********************************************************************
 *%FUNCTION: chomp
@@ -227,39 +243,6 @@ chomp(char *str)
     *s = 0;
 }
 
-/**********************************************************************
-* %FUNCTION: safeWriteHeader
-* %ARGUMENTS:
-*  fd -- file to write to
-*  str -- a string value
-* %RETURNS:
-*  0 if header seems OK; 1 if suspicious character found.
-*  Note that "str" MAY BE modified!
-* %DESCRIPTION:
-*  Writes "str" to FP with the following changes:
-*    CR   -> written as space
-***********************************************************************/
-int
-safeWriteHeader(int fd,
-		unsigned char *str)
-{
-    int suspicious = 0;
-    unsigned char *origStr = str;
-
-    for(; *str; str++) {
-	/* Do not write \r to header file -- convert to space*/
-	if (*str == '\r') {
-	    if (*(str+1) != '\n') {
-		suspicious = 1;
-		*str = ' ';
-	    }
-	}
-    }
-
-    writestr(fd, origStr);
-    return suspicious;
-}
-
 /**********************************************************************
 * %FUNCTION: MXCommand
 * %ARGUMENTS:
@@ -369,55 +352,6 @@ MXScanDir(char const *sockname,
     return MD_TEMPFAIL;
 }
 
-/**********************************************************************
-* %FUNCTION: write_percent_encoded
-* %ARGUMENTS:
-*  str -- a string
-*  fd -- file to write to
-* %RETURNS:
-*  Nothing
-* %DESCRIPTION:
-*  Writes a string to a file, escaping the following chars:
-*  <= ascii-32; > ascii-126, %, \, ', ".  These are replaced with "%XY"
-*  where X and Y are hex digits.
-* "
-***********************************************************************/
-void
-write_percent_encoded(unsigned char const *str,
-		      int fd)
-{
-    unsigned char buf[2048];
-    int c;
-    unsigned char *s = buf;
-    if (strlen(str) < 640) {
-	/* Fast -- create in memory buffer, then write out all at once */
-	while ((c = *str++) != 0) {
-	    if (c <= 32 || c > 126 || c == '%' ||
-		c == '\\' || c == '\'' || c == '"') {
-		sprintf(s, "%%%02X", (unsigned int) c);
-		s += 3;
-	    } else {
-		*s++ = c;
-	    }
-	}
-	/* Null termination -- not strictly required because of writen */
-	*s = 0;
-	writen(fd, buf, s-buf);
-    } else {
-	/* Slow -- string too long; use individual writes */
-	while ((c = *str++) != 0) {
-	    if (c <= 32 || c > 126 || c == '%' ||
-		c == '\\' || c == '\'' || c == '"') {
-		sprintf(s, "%%%02X", (unsigned int) c);
-		writen(fd, s, 3);
-	    } else {
-		*s = c;
-		writen(fd, s, 1);
-	    }
-	}
-    }
-}
-
 /**********************************************************************
 * %FUNCTION: percent_decode
 * %ARGUMENTS:
@@ -428,10 +362,10 @@ write_percent_encoded(unsigned char const *str,
 *  Decodes buf IN PLACE.
 ***********************************************************************/
 void
-percent_decode(unsigned char *buf)
+percent_decode(char *buf)
 {
-    unsigned char *in = buf;
-    unsigned char *out = buf;
+    unsigned char *in = (unsigned char *) buf;
+    unsigned char *out = (unsigned char *) buf;
     unsigned int val;
 
     if (!buf) {
@@ -440,7 +374,7 @@ percent_decode(unsigned char *buf)
 
     while(*in) {
 	if (*in == '%' && isxdigit(*(in+1)) && isxdigit(*(in+2))) {
-	    sscanf(in+1, "%2x", &val);
+	    sscanf((char *) in+1, "%2x", &val);
 	    *out++ = (unsigned char) val;
 	    in += 3;
 	    continue;
@@ -465,42 +399,44 @@ percent_decode(unsigned char *buf)
 *  trailing 0.
 ***********************************************************************/
 int
-percent_encode(unsigned char *in,
-	       unsigned char *out,
+percent_encode(char const *in,
+	       char *out,
 	       int outlen)
 {
     unsigned char tmp[8];
     int nwritten = 0;
     unsigned char c;
+    unsigned char const *uin = (unsigned char const *) in;
+    unsigned char *uout = (unsigned char *) out;
 
     if (outlen <= 0) {
 	return 0;
     }
     if (outlen == 1) {
-	*out = 0;
+	*uout = 0;
 	return 0;
     }
 
     /* Do real work */
-    while((c = *in++) != 0) {
+    while((c = *uin++) != 0) {
 	if (c <= 32 || c > 126 || c == '%' || c == '\\' || c == '\'' || c == '"') {
 	    if (nwritten >= outlen-3) {
 		break;
 	    }
 	    sprintf((char *) tmp, "%%%02X", (unsigned int) c);
-	    *out++ = tmp[0];
-	    *out++ = tmp[1];
-	    *out++ = tmp[2];
+	    *uout++ = tmp[0];
+	    *uout++ = tmp[1];
+	    *uout++ = tmp[2];
 	    nwritten += 3;
 	} else {
-	    *out++ = c;
+	    *uout++ = c;
 	    nwritten++;
 	}
 	if (nwritten >= outlen-1) {
 	    break;
 	}
     }
-    *out = 0;
+    *uout = 0;
     return nwritten;
 }
 
@@ -541,7 +477,7 @@ percent_encode_command(int term_with_newline, char *out, int outlen, ...)
 	    *out++ = ' ';
 	    spaceleft--;
 	}
-	len = percent_encode((unsigned char *) arg, (unsigned char *) out, spaceleft);
+	len = percent_encode(arg, out, spaceleft);
 	spaceleft -= len;
 	out += len;
     }
@@ -737,7 +673,7 @@ MXSenderOK(char const *sockname,
     /* Append ESMTP args */
     l = strlen(cmd);
     for (i=1; sender_argv[i]; i++) {
-	percent_encode((unsigned char *) sender_argv[i],
+	percent_encode(sender_argv[i],
 		       ans,
 		       sizeof(ans));
 	l2 = strlen(ans) + 1;
@@ -832,7 +768,7 @@ MXRecipientOK(char const *sockname,
     /* Append ESMTP args */
     l = strlen(cmd);
     for (i=1; recip_argv[i]; i++) {
-	percent_encode((unsigned char *) recip_argv[i],
+	percent_encode(recip_argv[i],
 		       ans,
 		       sizeof(ans));
 	l2 = strlen(ans) + 1;
@@ -852,31 +788,6 @@ MXRecipientOK(char const *sockname,
     return munch_mx_return(ans, msg);
 }
 
-/**********************************************************************
-* %FUNCTION: write_mx_command
-* %ARGUMENTS:
-*  fd -- file to write to
-*  cmd -- the command to write.  A single character.
-*  buf -- command arguments
-* %RETURNS:
-*  Nothing
-* %DESCRIPTION:
-*  Writes a command for communicating with filter.
-***********************************************************************/
-void
-write_mx_command(int fd,
-		 char cmd,
-		 unsigned char const *buf)
-{
-    unsigned char newline = '\n';
-    if (fd < 0) return;
-    writen(fd, (unsigned char *) &cmd, 1);
-    if (buf) {
-	write_percent_encoded(buf, fd);
-    }
-    writen(fd, &newline, 1);
-}
-
 /**********************************************************************
 * %FUNCTION: writen
 * %ARGUMENTS:
@@ -890,7 +801,7 @@ write_mx_command(int fd,
 ***********************************************************************/
 int
 writen(int fd,
-       unsigned char const *buf,
+       char const *buf,
        size_t len)
 {
     int r;
@@ -929,7 +840,7 @@ writen(int fd,
 ***********************************************************************/
 int
 writestr(int fd,
-	 unsigned char const *buf)
+	 char const *buf)
 {
     return writen(fd, buf, strlen(buf));
 }
@@ -949,7 +860,7 @@ int
 readn(int fd, void *buf, size_t count)
 {
     size_t num_read = 0;
-    unsigned char *c = (unsigned char *) buf;
+    char *c = (char *) buf;
     while (count) {
 	int n = read(fd, c, count);
 	if (n == 0) { /* EOF */
@@ -1056,6 +967,38 @@ validate_smtp_dsn(char const *dsn,
     return 1;
 }
 
+/**********************************************************************
+* %FUNCTION: remove_local_socket
+* %ARGUMENTS:
+*  str -- a string of the form:
+*                   /path/to/sock   (assumed to be unix:/path/to/sock)
+*                   unix:/path/to/sock
+*                   local:/path/to/sock
+*                   inet:port  (host defaults to LOOPBACK)
+*                   inet_any:port (host defaults to INADDR_ANY)
+* %RETURNS:
+*  Whatever remove() returns if it's a local socket; otherwise zero
+* %DESCRIPTION:
+*  If the socket is local, then it's removed from the file system
+***********************************************************************/
+int
+remove_local_socket(char const *str)
+{
+    char const *path;
+
+    if (*str == '/') {
+	path = str;
+    } else if (!strncmp(str, "unix:", 5)) {
+	path = str+5;
+    } else if (!strncmp(str, "local:", 6)) {
+	path = str+6;
+    } else {
+	/* Not unix-domain socket */
+	return 0;
+    }
+    return remove(path);
+}
+
 /**********************************************************************
 * %FUNCTION: make_listening_socket
 * %ARGUMENTS:
@@ -1284,3 +1227,29 @@ is_localhost(struct sockaddr *sa)
 #endif
     return 0;
 }
+
+#ifdef ENABLE_DEBUGGING
+void *
+malloc_debug(void *ctx, size_t x, char const *fname, int line)
+{
+    void *ptr = (malloc) (x);
+    syslog(LOG_DEBUG, "%p: %s(%d): malloc(%lu) = %p\n", ctx, fname,
+	   line, (unsigned long) x, ptr);
+    return ptr;
+}
+
+char *
+strdup_debug(void *ctx, char const *s, char const *fname, int line)
+{
+    char *dup = (strdup) (s);
+    syslog(LOG_DEBUG, "%p: %s(%d): strdup(\"%.25s\") = %p\n", ctx, fname, line, s, dup);
+    return dup;
+}
+
+void
+free_debug(void *ctx, void *x, char const *fname, int line)
+{
+    syslog(LOG_DEBUG, "%p: %s(%d): free(%p)\n", ctx, fname, line, x);
+    (free)(x);
+}
+#endif
